@@ -4,11 +4,10 @@ The homepage of the KMC project is http://sun.aei.polsl.pl/kmc
 
 Authors: Marek Kokot
 
-Version: 3.1.1
-Date   : 2019-05-19
+Version: 3.2.1
+Date   : 2022-01-04
 */
 
-#include "stdafx.h"
 #include "fastq_filter.h"
 #include <numeric>
 
@@ -19,7 +18,7 @@ using namespace std;
 /*****************************************************************************************************************************/
 
 /*****************************************************************************************************************************/
-CFastqFilter::CFastqFilter(CFilteringParams& Params, CFilteringQueues& Queues, CKMCFile& kmc_api) :
+CFastqFilter::CFastqFilter(CFilteringParams& Params, CFilteringQueues& Queues, CKffAndKMCRandomAccess& kmc_api) :
 kmc_api(kmc_api)
 {
 	input_part_queue = Queues.input_part_queue;
@@ -39,7 +38,7 @@ kmc_api(kmc_api)
 }
 
 /*****************************************************************************************************************************/
-CWFastqFilter::CWFastqFilter(CFilteringParams& Params, CFilteringQueues& Queues, CKMCFile& kmc_api)
+CWFastqFilter::CWFastqFilter(CFilteringParams& Params, CFilteringQueues& Queues, CKffAndKMCRandomAccess& kmc_api)
 {
 	ff = make_unique<CFastqFilter>(Params, Queues, kmc_api);
 }
@@ -594,6 +593,7 @@ public:
 		return true;
 	}
 };
+
 class CFastqFilter::HardMaskFastqToFastaHelper
 {
 	CFastqFilter& owner;
@@ -621,6 +621,7 @@ public:
 		return true;
 	}
 };
+
 class CFastqFilter::HardMaskFastaToFastaHelper
 {
 	CFastqFilter& owner;
