@@ -4,14 +4,14 @@
   
   Authors: Marek Kokot
   
-  Version: 3.1.1
-  Date   : 2019-05-19
+  Version: 3.2.1
+  Date   : 2022-01-04
 */
 
 #ifndef _THREAD_WATCH_H
 #define _THREAD_WATCH_H
 
-#ifdef WIN32
+#ifdef _WIN32
 
 class CThreadWatchImpl;
 
